-- Do graph stuff
-- This should be a separate addon someday

--[[
Name: GraphHandler-1.0
Author(s): Sutorix <sutorix@hotmail.com>
Description: A library with common graph routines used by FB_TrackingFrame.
--]]

local MAJOR_VERSION = "GraphHandler-1.0"
local MINOR_VERSION = 3

if not LibStub then error(MAJOR_VERSION .. " requires LibStub") end

local GraphHandler, oldLib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not GraphHandler then
	return
end

GraphHandler.BAR = "BAR";
GraphHandler.LEGEND = "LEGEND";
GraphHandler.OVERLAY = "OVERLAY";
GraphHandler.LINE = "LINE";
GraphHandler.LABEL = "LABEL";
GraphHandler.TEXT = "TEXT";

function GraphHandler:OnClick()
   if ( self.ClickHandler ) then
       self.ClickHandler(self, button);
   end
end


local function MakeName(self, kind, idx)
   if ( not idx ) then
      idx = table.getn(self.elements[kind])+1;
   end
   return self:GetName().."_"..kind.."_"..idx;
end

GraphHandler.MakeElement = {};

local function Legend_OnEnter(self)
   if( self.item or self.tooltip ) then
      GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
      if (self.item and self.item ~= "") then
         GameTooltip:SetHyperlink("item:"..self.item);
      elseif (self.tooltip) then
         GameTooltip:SetText(self.tooltip);
      end
   end
end

local function Legend_OnLeave(self)
   if( self.item or self.tooltop ) then
      self.updateToolTip = nil;
      GameTooltip:Hide();
   end
end

local function Legend_OnClick(self, button)
   if ( button == "LeftButton" and IsShiftKeyDown() and self.item ) then
       FishingBuddy.ChatLink(self.item, self.name, self.color);
   elseif ( self.ClickHandler ) then
       self:ClickHandler(button);
   end
end

GraphHandler.MakeElement[GraphHandler.LEGEND] = function(self)
   local name = MakeName(self, GraphHandler.LEGEND);
   local legend = CreateFrame("Button", name, self);
   legend:SetWidth(30);
   legend:SetHeight(18);
   local bar = CreateFrame("Button", name.."Color", legend);
   local c = bar:CreateTexture(name.."ColorTexture", "OVERLAY");
   local icon = CreateFrame("Button", name.."Icon", legend);
   local tex = icon:CreateTexture(name.."IconTexture", "BORDER");
   icon:SetWidth(16);
   icon:SetHeight(16);
   icon:SetPoint("LEFT", c, "RIGHT", 2, 0);
   legend:SetScript("OnEnter", Legend_OnEnter);
   legend:SetScript("OnLeave", Legend_OnLeave);
   legend:SetScript("OnClick", Legend_OnClick);
   return legend;
end

GraphHandler.MakeElement[GraphHandler.BAR] = function(self)
   local kind = GraphHandler.BAR;
   local name = MakeName(self, kind);
   local bar = CreateFrame("Button", name, self);
   local c = bar:CreateTexture(name.."Texture", "OVERLAY");
   c:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ReputationBar");
   return bar;
end

GraphHandler.MakeElement[GraphHandler.LINE] = function(self)
   local kind = GraphHandler.LINE;
   local name = MakeName(self, kind);
   local c = self:CreateTexture(name.."Texture", "OVERLAY");
   c:SetTexture("Interface\\TaxiFrame\\UI-Taxi-Line");
   return c;
end

GraphHandler.MakeElement[GraphHandler.TEXT] = function(self)
   local name = MakeName(self, GraphHandler.TEXT);
   local text = CreateFrame("Button", name, self);
   _ = text:CreateFontString(name.."T", "OVERLAY", "GameFontNormalSmall");
   return text;
end

GraphHandler.MakeElement[GraphHandler.OVERLAY] = function(self)
   local name = MakeName(self, GraphHandler.OVERLAY);
   local over = CreateFrame("Button", name, self);
   over:SetScript("OnClick", GraphHandler.OnClick);
end

function GraphHandler:GetElement(kind, idx)
   local n = table.getn(self.elements[kind]);
   if ( idx <= n ) then
      return self.elements[kind][idx];
   else
      local e;
      while (n < idx) do
         e = GraphHandler.MakeElement[kind](self);
         tinsert(self.elements[kind], e);
         n = n + 1;
      end
      return e;
   end
end

local function SetColorBar(bar, base, x, y, width, height, r, g, b)
   local name;
   if ( type(bar) == "string" ) then
      name = bar;
      bar = getglobal(name);
   elseif ( bar ) then
      name = bar:GetName();
   end
   if ( bar ) then
      if ( not r ) then
         r = 1.0;
         g = 1.0;
         b = 1.0;
      end
      if ( base ) then
         local tex = getglobal(name.."Texture");
         if ( x and y ) then
            bar:ClearAllPoints();
            bar:SetPoint("BOTTOMLEFT", base, "BOTTOMLEFT", x, y);
         end
         if ( width ) then
            bar:SetWidth(width);
         end
         if ( height ) then
            bar:SetHeight(height);
         end
         tex:SetVertexColor(r, g, b);
         tex:Show();
         bar:Show();
      else
         bar:Hide();
      end
   end
end

local function InitElements(self)
   if ( not self.elements ) then
      self.elements = {};
      self.elements[GraphHandler.BAR] = {};
      self.elements[GraphHandler.LINE] = {};
      self.elements[GraphHandler.LEGEND] = {};
      self.elements[GraphHandler.OVERLAY] = {};
      self.elements[GraphHandler.LABEL] = {};
      self.elements[GraphHandler.TEXT] = {};
   end
end

-- Registration functions

function GraphHandler:Register(frame, kind)
   if ( not self.elements[kind] ) then
       self.elements[kind] = {};
    end
    tinsert(self.elements[kind], self);
    return #self.elements[kind];
end

function GraphHandler:ClearPlot()
   for _,elem in pairs({ GraphHandler.BAR, GraphHandler.LEGEND }) do
      for idx,what in pairs(self.elements[elem]) do
         what:Hide();
      end
   end
end

function GraphHandler:PlotData(data, offset, skip, r, g, b)
  local limit = table.getn(data);
  local start = 1;
  if ( data[0] ) then
     start = 0;
     limit = limit - 1;
  end

  local bdx=offset+1;
  local bwid = self.barWidth;
  local bspc = self.barSpacing;
  local maxval = self.maxVal;
  local xoff = offset*bspc;
  local yoff = 0;
  local graphHeight = self:GetHeight();
  for idx=start,limit,1 do
     local bar = self:GetElement(GraphHandler.BAR, bdx);
     bdx = bdx + skip + 1;
     if ( bar ) then
        if ( (maxval > 0) and (data[idx] > 0) ) then
           local height = (data[idx]*graphHeight)/maxval;
           SetColorBar(bar, self, xoff, 0, bwid, height, r, g, b);
        else
           bar:Hide();
        end
     end
     xoff = xoff + bwid + skip*bspc;
  end
  return xoff - skip*bspc;
end

function GraphHandler:UpdateOverlay()
   local ol = self:GetElement(GraphHandler.OVERLAY, 1);
   if ( ol ) then
      local legends = self.elements[GraphHandler.LEGEND];
      local settop = true;
      local bottom = 0;
      ol:Hide();
      for idx=1,table.getn(legends),1 do
         local legend = legends[idx];
         if ( legend:IsVisible() ) then
            if ( settop ) then
               ol:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, -(idx-1)*21);
               ol:Show();
               settop = false;
            end
            bottom = bottom + 21;
         end
      end
      if ( bottom > 0) then
         ol:SetHeight(bottom);
      end
   end
end

function GraphHandler:PlotLegend(ldx, name, item, texture, r, g, b)
   local legend = self:GetElement(GraphHandler.LEGEND, ldx);
   if ( legend ) then
      local legname = legend:GetName();
      local icon = getglobal(legname.."Icon");
      local tex = getglobal(legname.."IconTexture");
      local color = getglobal(legname.."Color");
      SetColorBar(color, legend, 0, 1, 10, 16, r, g, b);
      color:Show();
      tex:SetTexture(texture);
      icon:Show();
      legend:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, -(ldx-1)*21);
      legend.item = item;
      legend.name = name;
      legend:Show();
      self:UpdateOverlay();
   end
end

-- function DrawRouteLine(T, C, sx, sy, ex, ey, w, relPoint)

function GraphHandler:PlotGrid(label, texth, width, textv, height)
   local ldx=1;
   local tdx=1;
   if ( not width ) then
      width = self:GetWidth();
   end
   if ( not height ) then
      height = self:GetHeight();
   end
   if ( label ) then
      local button = self:GetElement(GraphHandler.TEXT, tdx);
      if ( button ) then
         local text = getglobal(button:GetName().."T");
         text:SetText(label);
         text:SetPoint("BOTTOM", self, "BOTTOM", 0, -28);
         text:SetVertexColor(1.0, 1.0, 1.0);
         text:Show();
         tdx = tdx + 1;
      end
   end
   if ( texth ) then
      local linesh = 0;
      for _,_ in pairs(texth) do
         linesh = linesh + 1;
      end
      local offset = height/(linesh-1);
      for label,tick in pairs(texth) do
         local line = self:GetElement(GraphHandler.LINE, ldx);
         if ( line ) then
            ldx = ldx + 1;
            line:SetAlpha(0.5);
            DrawRouteLine(line, self, 0, tick*offset, width, tick*offset, 16);

            local button = self:GetElement(GraphHandler.TEXT, tdx);
            if ( button ) then
               local text = getglobal(button:GetName().."T");
               if ( type(label) ~= "string" ) then
                  label = string.format("%3d", label);
               end
               text:SetText(label);
               text:SetPoint("LEFT", line, "LEFT", -text:GetWidth(), 0);
               tdx = tdx + 1;
            end
         end
      end
   end
   if ( textv ) then
      local linesv = 0;
      for _,_ in pairs(textv) do
         linesv = linesv + 1;
      end
      local offset = self.barSpacing + self.barWidth;
      for label,tick in pairs(textv) do
         local line = self:GetElement(GraphHandler.LINE, ldx);
         if ( line ) then
            local xpos = tick*offset;
            ldx = ldx + 1;
            line:SetAlpha(0.5);
            DrawRouteLine(line, self, xpos, 0, xpos, height, 16);
            local button = self:GetElement(GraphHandler.TEXT, tdx);
            if ( button ) then
               local text = getglobal(button:GetName().."T");
               if ( type(label) ~= "string" ) then
                  label = string.format("%3d", label);
               end
               text:SetText(label);
               local wid = text:GetWidth();
               if ( xpos < wid ) then
                  text:SetPoint("TOPLEFT", line, "BOTTOMLEFT", 0, -4);
               elseif ( xpos > (width - wid) ) then
                  text:SetPoint("TOPRIGHT", line, "BOTTOMRIGHT", 0, -4);
               else
                  text:SetPoint("TOP", line, "BOTTOM", 0, -4);
               end
               text:Show();
               tdx = tdx + 1;
            end
         end
      end
   end
end

function GraphHandler:SetOverlayClick(onclick)
   local ol = self:GetElement(GraphHandler.OVERLAY, 1);
   if ( ol ) then
      ol.ClickHandler = onclick;
      ol:SetFrameLevel(self:GetFrameLevel() + 1);
      return true;
   end
-- return nil;
end

function GraphHandler:SetLegendClick(onclick)
   local legends = self.elements[GraphHandler.LEGEND];
   if ( legends ) then
      local n = table.getn(legends);
      if ( n > 0 ) then
         for idx=1,n,1 do
            local legend = legends[idx];
            legend:SetFrameLevel(self:GetFrameLevel() + 1);
            legend.OnClickHandler = onclick; 
         end
         return true;
      end
   end
-- return nil;
end

function GraphHandler:NewGraph(name, setPoint, parent, relativePoint, relx, rely)
   local graph = CreateFrame("Frame", name, parent);
   graph:SetPoint(setPoint, parent, relativePoint, relx, rely);
   InitElements(graph);
   graph.Register = GraphHandler.Register;
   graph.ClearPlot = GraphHandler.ClearPlot;
   graph.GetElement = GraphHandler.GetElement;
   graph.UpdateOverlay = GraphHandler.UpdateOverlay;
   graph.SetLegendClick = GraphHandler.SetLegendClick;
   graph.SetOverlayClick = GraphHandler.SetOverlayClick;
   graph.PlotData = GraphHandler.PlotData;
   graph.PlotLegend = GraphHandler.PlotLegend;
   graph.PlotGrid = GraphHandler.PlotGrid;
   return graph;
end

